#!/usr/bin/python
# -*- coding: utf-8 -*-
# Archiveur, a simple IRC log bot
# catwell <catwell@archlinux.us> 2007.12 (reworked 2008.12)

import time, irclib, ConfigParser
from irclib import IRC, nm_to_n
import __main__ as main

config = "archiveur.conf"

parser = ConfigParser.SafeConfigParser()
parser.read(config)

# parameters
network = parser.get("shared", "network")
port = int(parser.get("bot", "port"))
channel = '#'+parser.get("shared", "channel")
nick = parser.get("bot", "nick")
password = parser.get("bot", "password")
admins=parser.get("bot", "admins").split(',')
logdir = parser.get("shared", "logdir")+'/'
log = 1

# functions
def quit():
  tolog(archimsg('disconnection'))
  cnx.quit('-- admin request --')
  exit(0)

def setlog(x):
  global log
  log = x

def tolog(x):
  global log
  if log:
    log = open(logdir+time.strftime('%Y%m%d',time.localtime()),'a')
    log.write(time.strftime('[%H:%M:%S] ',time.localtime())+x+"\n")
    log.close()

def archimsg(x):
  return '=> Archiveur: '+x+' <='

# handlers

def on_ctcp(cnx, event):
  n = nm_to_n(event.source())
  if event.arguments()[0] == 'VERSION':
   cnx.privmsg(n, nick+': IRC bot for '+channel+'.')

def on_invite(cnx, event):
  cnx.privmsg(nm_to_n(event.source()), "No thanks, I'd better not join "+event.arguments()[0]+'.')

def on_join(cnx, event):
  n = nm_to_n(event.source())
  if n == nick:
    tolog(archimsg('connection'))
  else:
    tolog(n+' ('+event.source()+') joined the channel.')

def on_kick(cnx,event):
  n = nm_to_n(event.source())
  cible = event.arguments()[0]
  if cible == nick:
    tolog(archimsg('kicked by '+n))
    cnx.join(channel)
    cnx.privmsg(channel, "I'm back!")
  else:
    tolog(n+' kicked '+cible+'.')

def on_mode(cnx,event):
  tolog(nm_to_n(event.source())+' set mode '+event.arguments()[0]+"%s." % (len(event.arguments())-1 and ' on '+', '.join(event.arguments()[1:]) or ''))

def on_nick(cnx, event):
  tolog(nm_to_n(event.source())+' changed his nick to '+event.target()+'.')

def on_other(cnx, event):
  pass
  # unsup = open(logdir+'unsup.log','a')
  # unsup.write(time.strftime("[%H:%M:%S]\n",time.localtime())+str(event.arguments())+"\n"+str(event.source())+"\n"+str(event.target())+"\n"+str(event.eventtype())+"\n\n")
  # unsup.close()

def on_part(cnx, event):
  n = nm_to_n(event.source())
  if n == nick:
    tolog(archimsg('disconnection'))
  else:
    tolog(n+' left the channel ('+event.arguments()[0]+').')

def on_privmsg(cnx, event):
  n = nm_to_n(event.source())
  if n in admins:
    msg = event.arguments()[0]
    if msg[0] == '!':
      if msg[1:] == 'bye':
        quit()
      elif msg[1:] == 'log':
        setlog(1)
      elif msg[1:] == '!log':
        setlog(0)
    elif msg[0] == '*':
      cnx.privmsg(channel, msg[1:])
      tolog('<'+nick+'> '+msg[1:])
  else:
    cnx.privmsg(n, "I'm busy logging, don't disturb me please!")

def on_pubmsg(cnx, event):
  tolog('<'+nm_to_n(event.source())+'> '+event.arguments()[0])

def on_quit(cnx, event):
  n = nm_to_n(event.source())
  if n == nick:
    tolog(archimsg('disconnection'))
  else:
    tolog(n+' left IRC ('+event.arguments()[0]+').')

# creation
irc = IRC()
cnx = irc.server()
cnx.connect(network, port, nick, password)
cnx.join(channel)

# events
for x in ['ctcp', 'invite', 'join', 'kick', 'mode', 'nick', 'part', 'privmsg', 'pubmsg', 'quit']:
 cnx.add_global_handler(x, getattr(main, "on_" + x))

for x in ['error', 'ping', 'privnotice', 'pubnotice', 'pong', 'dccmsg', 'ctcpreply', 'dcc_connect']: # not handled
 cnx.add_global_handler(x, getattr(main, "on_other"))
 
# main loop
irc.process_forever()
